@@ -7,8 +7,7 @@ from typing import Dict, Optional
 
 from daytona_api_client_async import PortPreviewUrl
 from daytona_api_client_async import Sandbox as SandboxDto
-from daytona_api_client_async import SandboxApi, ToolboxApi
-from daytona_api_client_async.exceptions import BadRequestException
+from daytona_api_client_async import SandboxApi, SshAccessDto, SshAccessValidationDto, ToolboxApi
 from pydantic import ConfigDict, PrivateAttr
 
 from .._utils.errors import intercept_errors
@@ -55,6 +54,8 @@ class AsyncSandbox(SandboxDto):
         build_info (str): Build information for the Sandbox if it was created from dynamic build.
         created_at (str): When the Sandbox was created.
         updated_at (str): When the Sandbox was last updated.
+        network_block_all (bool): Whether to block all network access for the Sandbox.
+        network_allow_list (str): Comma-separated list of allowed CIDR network addresses for the Sandbox.
     """
 
     _fs: AsyncFileSystem = PrivateAttr()
@@ -90,7 +91,7 @@ class AsyncSandbox(SandboxDto):
 
         self._fs = AsyncFileSystem(self.id, toolbox_api, self.__get_root_dir)
         self._git = AsyncGit(self.id, toolbox_api, self.__get_root_dir)
-        self._process = AsyncProcess(self.id, code_toolbox, toolbox_api, self.__get_root_dir)
+        self._process = AsyncProcess(self.id, code_toolbox, toolbox_api, self.__get_root_dir, self.get_preview_link)
         self._computer_use = AsyncComputerUse(self.id, toolbox_api)
 
     @property
@@ -216,17 +217,10 @@ class AsyncSandbox(SandboxDto):
             ```
         """
         start_time = time.time()
-        try:
-            sandbox = await self._sandbox_api.start_sandbox(self.id, _request_timeout=timeout or None)
-            self.__process_sandbox_dto(sandbox)
-        except BadRequestException as e:
-            if "Sandbox failed to start: Timeout after" in str(e.body):
-                await self.refresh_data()
-            else:
-                raise e
-
+        sandbox = await self._sandbox_api.start_sandbox(self.id, _request_timeout=timeout or None)
+        self.__process_sandbox_dto(sandbox)
         time_elapsed = time.time() - start_time
-        await self.wait_for_sandbox_start(timeout=max(0.1, timeout - time_elapsed) if timeout else None)
+        await self.wait_for_sandbox_start(timeout=max(0.001, timeout - time_elapsed) if timeout else timeout)
 
     @intercept_errors(message_prefix="Failed to stop sandbox: ")
     @with_timeout(
@@ -250,9 +244,11 @@ class AsyncSandbox(SandboxDto):
             print("Sandbox stopped successfully")
             ```
         """
+        start_time = time.time()
         await self._sandbox_api.stop_sandbox(self.id, _request_timeout=timeout or None)
         await self.refresh_data()
-        await self.wait_for_sandbox_stop()
+        time_elapsed = time.time() - start_time
+        await self.wait_for_sandbox_stop(timeout=max(0.001, timeout - time_elapsed) if timeout else timeout)
 
     @intercept_errors(message_prefix="Failed to remove sandbox: ")
     async def delete(self, timeout: Optional[float] = 60) -> None:
@@ -311,6 +307,7 @@ class AsyncSandbox(SandboxDto):
         """Waits for the Sandbox to reach the 'stopped' state. Polls the Sandbox status until it
         reaches the 'stopped' state, encounters an error or times out. It will wait up to 60 seconds
         for the Sandbox to stop.
+        Treats destroyed as stopped to cover ephemeral sandboxes that are automatically deleted after stopping.
 
         Args:
             timeout (Optional[float]): Maximum time to wait in seconds. 0 means no timeout. Default is 60 seconds.
@@ -318,7 +315,7 @@ class AsyncSandbox(SandboxDto):
         Raises:
             DaytonaError: If timeout is negative. If Sandbox fails to stop or times out.
         """
-        while self.state != "stopped":
+        while self.state not in ["stopped", "destroyed"]:
             try:
                 await self.refresh_data()
 
@@ -446,6 +443,33 @@ class AsyncSandbox(SandboxDto):
         await self._sandbox_api.archive_sandbox(self.id)
         await self.refresh_data()
 
+    @intercept_errors(message_prefix="Failed to create SSH access: ")
+    async def create_ssh_access(self, expires_in_minutes: Optional[int] = None) -> SshAccessDto:
+        """Creates an SSH access token for the sandbox.
+
+        Args:
+            expires_in_minutes (Optional[int]): The number of minutes the SSH access token will be valid for.
+        """
+        return (await self._sandbox_api.create_ssh_access(self.id, expires_in_minutes=expires_in_minutes)).data
+
+    @intercept_errors(message_prefix="Failed to revoke SSH access: ")
+    async def revoke_ssh_access(self, token: str) -> None:
+        """Revokes an SSH access token for the sandbox.
+
+        Args:
+            token (str): The token to revoke.
+        """
+        await self._sandbox_api.revoke_ssh_access(self.id, token)
+
+    @intercept_errors(message_prefix="Failed to validate SSH access: ")
+    async def validate_ssh_access(self, token: str) -> SshAccessValidationDto:
+        """Validates an SSH access token for the sandbox.
+
+        Args:
+            token (str): The token to validate.
+        """
+        return (await self._sandbox_api.validate_ssh_access(token)).data
+
     async def __get_root_dir(self) -> str:
         if not self._root_dir:
             self._root_dir = await self.get_user_root_dir()
@@ -476,3 +500,5 @@ class AsyncSandbox(SandboxDto):
         self.build_info = sandbox_dto.build_info
         self.created_at = sandbox_dto.created_at
         self.updated_at = sandbox_dto.updated_at
+        self.network_block_all = sandbox_dto.network_block_all
+        self.network_allow_list = sandbox_dto.network_allow_list
